@@ -19,7 +19,7 @@
     Taking the sum, the sum of squares, ..., upto the sum of x^5
 
 >>> import Data.Traversable
->>> let powerSums = sequenceA [premap (^n) L.sum | n <- [1..5]]
+>>> let powerSums = sequenceA [L.premap (^n) L.sum | n <- [1..5]]
 >>> L.fold powerSums [1..10]
 [55,385,3025,25333,220825]
 
@@ -64,7 +64,9 @@ module Control.Foldl (
     , sum
     , product
     , maximum
+    , maximumBy
     , minimum
+    , minimumBy
     , elem
     , notElem
     , find
@@ -106,6 +108,8 @@ module Control.Foldl (
     , EndoM(..)
     , HandlerM
     , handlesM
+    , folded
+    , filtered
 
     -- * Re-exports
     -- $reexports
@@ -116,12 +120,12 @@ module Control.Foldl (
 
 import Control.Applicative
 import Control.Foldl.Internal (Maybe'(..), lazy, Either'(..), hush)
-import Control.Monad ((>=>))
+import Control.Monad ((<=<))
 import Control.Monad.Primitive (PrimMonad, RealWorld)
 import Control.Comonad
 import Data.Foldable (Foldable)
-import Data.Functor.Constant (Constant(Constant, getConstant))
 import Data.Functor.Identity (Identity, runIdentity)
+import Data.Functor.Contravariant (Contravariant(..))
 import Data.Monoid
 import Data.Profunctor
 import Data.Sequence ((|>))
@@ -169,7 +173,7 @@ data Pair a b = Pair !a !b
 
 instance Functor (Fold a) where
     fmap f (Fold step begin done) = Fold step begin (f . done)
-    {-# INLINABLE fmap #-}
+    {-# INLINE fmap #-}
 
 instance Profunctor Fold where
     lmap = premap
@@ -184,14 +188,14 @@ instance Comonad (Fold a) where
 
 instance Applicative (Fold a) where
     pure b    = Fold (\() _ -> ()) () (\() -> b)
-    {-# INLINABLE pure #-}
+    {-# INLINE pure #-}
 
     (Fold stepL beginL doneL) <*> (Fold stepR beginR doneR) =
         let step (Pair xL xR) a = Pair (stepL xL a) (stepR xR a)
             begin = Pair beginL beginR
             done (Pair xL xR) = doneL xL (doneR xR)
         in  Fold step begin done
-    {-# INLINABLE (<*>) #-}
+    {-# INLINE (<*>) #-}
 
 instance Monoid b => Monoid (Fold a b) where
     mempty = pure mempty
@@ -549,11 +553,33 @@ maximum :: Ord a => Fold a (Maybe a)
 maximum = _Fold1 max
 {-# INLINABLE maximum #-}
 
+{-| Computes the maximum element with respect to the given comparison
+    function
+-}
+maximumBy :: (a -> a -> Ordering) -> Fold a (Maybe a)
+maximumBy cmp = _Fold1 max'
+  where
+    max' x y = case cmp x y of
+        GT -> x
+        _  -> y
+{-# INLINABLE maximumBy #-}
+
 -- | Computes the minimum element
 minimum :: Ord a => Fold a (Maybe a)
 minimum = _Fold1 min
 {-# INLINABLE minimum #-}
 
+{-| Computes the minimum element with respect to the given comparison
+    function
+-}
+minimumBy :: (a -> a -> Ordering) -> Fold a (Maybe a)
+minimumBy cmp = _Fold1 min'
+  where
+    min' x y = case cmp x y of
+        GT -> y
+        _  -> x
+{-# INLINABLE minimumBy #-}
+
 {-| @(elem a)@ returns 'True' if the container has an element equal to @a@,
     'False' otherwise
 -}
@@ -773,17 +799,46 @@ vector = FoldM step begin done
     function to accept a 'Fold' or 'FoldM' using the 'purely' or 'impurely'
     combinators.
 
-    For example, the @pipes@ library implements a @foldM@ function in
+    For example, the @pipes@ library implements @fold@ and @foldM@ functions in
     @Pipes.Prelude@ with the following type:
 
-> foldM
+> Pipes.Prelude.fold
+>     :: Monad m
+>     -> (x -> a -> x) -> x -> (x -> b) -> Producer a m () -> m b
+>
+> Pipes.Prelude.foldM
 >     :: Monad m
 >     => (x -> a -> m x) -> m x -> (x -> m b) -> Producer a m () -> m b
 
-    @foldM@ is set up so that you can wrap it with 'impurely' to accept a
-    'FoldM' instead:
+    Both @fold@ and @foldM@ is set up so that you can wrap them with either
+    'purely' or 'impurely' to accept a 'Fold' or 'FoldM', respectively:
 
-> impurely foldM :: Monad m => FoldM m a b -> Producer a m () -> m b
+> purely Pipes.Prelude.fold
+>     :: Monad m => Fold a b -> Producer a m () -> m b
+>
+> impurely Pipes.Prelude.foldM
+>     :: Monad m => FoldM m a b -> Producer a m () -> m b
+
+    Similarly the @ofoldlUnwrap@ and @ofoldMUnwrap@ functions from the
+    @monotraversable@ package are written to interoperate with this library:
+
+> ofoldlUnwrap
+>     :: MonoFoldable mono
+>     => (x -> Element mono -> x) -> x -> (x -> b) -> mono -> b
+>
+> ofoldMUnwrap
+>     :: (Monad m, MonoFoldable mono)
+>     => (x -> Element mono -> m x) -> m x -> (x -> m b) -> mono -> m b
+
+    You can wrap these to accept 'Fold' or 'FoldM', too:
+
+> purely ofoldlUnwrap
+>     :: MonoFoldable mono
+>     => Fold (Element mono) b -> mono -> b
+>
+> impurely ofoldMUnwrap
+>     :: MonoFoldable mono
+>     => FoldM m (Element mono) b -> mono -> m b
 -}
 
 -- | Upgrade a fold to accept the 'Fold' type
@@ -919,7 +974,7 @@ premapM f (FoldM step begin done) = FoldM step' begin done
     Any lens, traversal, or prism will type-check as a `Handler`
 -}
 type Handler a b =
-    forall x . (b -> Constant (Endo x) b) -> a -> Constant (Endo x) a
+    forall x . (b -> Const (Dual (Endo x)) b) -> a -> Const (Dual (Endo x)) a
 
 {-| @(handles t folder)@ transforms the input of a `Fold` using a lens,
     traversal, or prism:
@@ -927,6 +982,7 @@ type Handler a b =
 > handles _1       :: Fold a r -> Fold (a, b) r
 > handles _Left    :: Fold a r -> Fold (Either a b) r
 > handles traverse :: Traversable t => Fold a r -> Fold (t a) r
+> handles folded   :: Foldable    t => Fold a r -> Fold (t a) r
 
 >>> fold (handles traverse sum) [[1..5],[6..10]]
 55
@@ -934,8 +990,8 @@ type Handler a b =
 >>> fold (handles (traverse.traverse) sum) [[Nothing, Just 2, Just 7],[Just 13, Nothing, Just 20]]
 42
 
->>> fold (handles (filtered even) sum) [1,3,5,7,21,21]
-42
+>>> fold (handles (filtered even) sum) [1..10]
+30
 
 >>> fold (handles _2 mconcat) [(1,"Hello "),(2,"World"),(3,"!")]
 "Hello World!"
@@ -951,26 +1007,26 @@ type Handler a b =
 handles :: Handler a b -> Fold b r -> Fold a r
 handles k (Fold step begin done) = Fold step' begin done
   where
-    step' = flip (appEndo . getConstant . k (Constant . Endo . flip step))
+    step' = flip (appEndo . getDual . getConst . k (Const . Dual . Endo . flip step))
 {-# INLINABLE handles #-}
 
 {-|
 > instance Monad m => Monoid (EndoM m a) where
 >     mempty = EndoM return
->     mappend (EndoM f) (EndoM g) = EndoM (f >=> g)
+>     mappend (EndoM f) (EndoM g) = EndoM (f <=< g)
 -}
 newtype EndoM m a = EndoM { appEndoM :: a -> m a }
 
 instance Monad m => Monoid (EndoM m a) where
     mempty = EndoM return
-    mappend (EndoM f) (EndoM g) = EndoM (f >=> g)
+    mappend (EndoM f) (EndoM g) = EndoM (f <=< g)
 
 {-| A Handler for the upstream input of `FoldM`
 
     Any lens, traversal, or prism will type-check as a `HandlerM`
 -}
 type HandlerM m a b =
-    forall x . (b -> Constant (EndoM m x) b) -> a -> Constant (EndoM m x) a
+    forall x . (b -> Const (Dual (EndoM m x)) b) -> a -> Const (Dual (EndoM m x)) a
 
 {-| @(handlesM t folder)@ transforms the input of a `FoldM` using a lens,
     traversal, or prism:
@@ -978,6 +1034,7 @@ type HandlerM m a b =
 > handlesM _1       :: FoldM m a r -> FoldM (a, b) r
 > handlesM _Left    :: FoldM m a r -> FoldM (Either a b) r
 > handlesM traverse :: Traversable t => FoldM m a r -> FoldM m (t a) r
+> handlesM folded   :: Foldable    t => FoldM m a r -> FoldM m (t a) r
 
     `handlesM` obeys these laws:
 
@@ -992,9 +1049,37 @@ type HandlerM m a b =
 handlesM :: Monad m => HandlerM m a b -> FoldM m b r -> FoldM m a r
 handlesM k (FoldM step begin done) = FoldM step' begin done
   where
-    step' = flip (appEndoM . getConstant . k (Constant . EndoM . flip step))
+    step' = flip (appEndoM . getDual . getConst . k (Const . Dual . EndoM . flip step))
 {-# INLINABLE handlesM #-}
 
+{-|
+> folded :: Foldable t => Fold (t a) a
+>
+> handles folded :: Foldable t => Fold a r -> Fold (t a) r
+-}
+folded
+    :: (Contravariant f, Applicative f, Foldable t)
+    => (a -> f a) -> (t a -> f (t a))
+folded k ts = contramap (\_ -> ()) (F.traverse_ k ts)
+
+{-|
+>>> fold (handles (filtered even) sum) [1..10]
+30
+
+>>> foldM (handlesM (filtered even) (mapM_ print)) [1..10]
+2
+4
+6
+8
+10
+
+-}
+filtered :: Monoid m => (a -> Bool) -> (a -> m) -> a -> m
+filtered p k x
+    | p x = k x
+    | otherwise = mempty
+{-# INLINABLE filtered #-}
+
 {- $reexports
     @Control.Monad.Primitive@ re-exports the 'PrimMonad' type class
 
